<?php
namespace SabaiApps\Directories\Component\Dashboard\DisplayButton;

use SabaiApps\Directories\Application;
use SabaiApps\Directories\Exception;
use SabaiApps\Directories\Component\Entity\Model\Bundle;
use SabaiApps\Directories\Component\Entity\Type\IEntity;

class PostDisplayButton extends AbstractPostDisplayButton
{    
    public function __construct(Application $application, $name)
    {
        $modal_danger = $allow_guest = false;
        switch ($name) {
            case 'dashboard_posts_edit':
                $route = 'edit';
                $allow_guest = true;
                break;
            case 'dashboard_posts_delete':
                $route = 'delete';
                $modal_danger = $allow_guest = true;
                break;
            case 'dashboard_posts_submit':
                $route = 'submit';
                break;
            default:
                throw new Exception\InvalidArgumentException();
        }
        parent::__construct($application, $name, $route, $modal_danger, $allow_guest);
    }
    
    protected function _displayButtonInfo(Bundle $bundle)
    {
        switch ($this->_name) {
            case 'dashboard_posts_edit':
                return array(
                    'label' => __('Edit post button', 'directories-frontend'),
                    'labellable' => false,
                    'default_settings' => array(
                        'modal' => false,
                        '_color' => 'outline-secondary',
                        '_icon' => 'fas fa-pen-to-square',
                    ),
                );
            case 'dashboard_posts_delete':
                return array(
                    'label' => __('Delete post button', 'directories-frontend'),
                    'labellable' => false,
                    'default_settings' => array(
                        '_color' => 'danger',
                        '_icon' => 'fas fa-trash-can',
                    ),
                );
            case 'dashboard_posts_submit':
                return array(
                    'label' => __('Submit post button', 'directories-frontend'),
                    'labellable' => false,
                    'default_settings' => array(
                        '_color' => 'outline-secondary',
                        '_icon' => 'fas fa-plus',
                    ),
                    'overlayable' => false,
                );
        }
    }

    public function displayButtonSettingsForm(Bundle $bundle, array $settings, $displayName, array $parents = [])
    {
        switch ($this->_name) {
            case 'dashboard_posts_edit':
                return [
                    'modal' => [
                        '#type' => 'checkbox',
                        '#title' => __('Show form in modal window', 'directories-frontend'),
                        '#default_value' => $settings['modal'],
                        '#horizontal' => true,
                    ],
                ];
        }
    }

    protected function _loadLinkAssets(Bundle $bundle, IEntity $entity, array $settings, $displayName)
    {
        if ($this->_name !== 'dashboard_posts_edit') {
            parent::_loadLinkAssets($bundle, $entity, $settings, $displayName);
        } else {
            if (!empty($settings['modal'])) {
                $this->_application->Entity_Form_loadAssets($bundle);
            }
        }
    }

    protected function _getLinkOptions(Bundle $bundle, IEntity $entity, array $settings, $displayName)
    {
        $options = parent::_getLinkOptions($bundle, $entity, $settings, $displayName);
        if ($this->_name === 'dashboard_posts_edit') {
            if (empty($settings['modal'])) {
                unset($options['container']);
            } else {
                $options['modalSize'] = 'xl';
            }
        }
        return $options;
    }

    protected function _getLinkUrlPath(Bundle $bundle, IEntity $entity, array $settings, $displayName)
    {
        return $this->_name === 'dashboard_posts_edit' ? '/' : parent::_getLinkUrlPath($bundle, $entity, $settings, $displayName);
    }
    
    protected function _getLabel(Bundle $bundle, IEntity $entity, array $settings)
    {
        switch ($this->_name) {
            case 'dashboard_posts_edit':
                return __('Edit', 'directories-frontend');
            case 'dashboard_posts_delete':
                return __('Delete', 'directories-frontend');
            case 'dashboard_posts_submit':
                return __('Submit for review', 'directories-frontend');
        }
    }
}